class AthinaMessages:
    """
    User facing messages.
    """

    SIGN_UP_FOR_BEST_EXPERIENCE = """
For the best experience, sign up at https://athina.ai and set an Athina API key.

See https://docs.athina.ai/evals/quick_start for more information.
"""
    
    NO_ATHINA_API_KEY = """
Please set an Athina API key.

See https://docs.athina.ai/evals/quick_start for more info.
    """

    NO_OPENAI_API_KEY = """
Please set an OpenAI API key.

See https://docs.athina.ai/evals/quick_start for more info.
    """

